@@ -2,41 +2,25 @@ use 5.006;
 use strict;
 use warnings;
 package Test::MinimumVersion;
+{
+  $Test::MinimumVersion::VERSION = '0.101081';
+}
 use base 'Exporter';
+# ABSTRACT: does your code require newer perl than you think?
 
-=head1 NAME
-
-Test::MinimumVersion - does your code require newer perl than you think?
-
-=head1 VERSION
-
-version 0.013
-
-=cut
-
-our $VERSION = '0.013';
-
-=head1 SYNOPSIS
-
-Example F<minimum-perl.t>:
-
-  #!perl
-  use Test::MinimumVersion;
-  all_minimum_version_ok('5.008');
-
-=cut
 
+use CPAN::Meta;
 use File::Find::Rule;
 use File::Find::Rule::Perl;
-use Perl::MinimumVersion 1.20;
-use YAML::Tiny;
-use version;
+use Perl::MinimumVersion 1.32; # numerous bugfies
+use version 0.70;
 
 use Test::Builder;
 @Test::MinimumVersion::EXPORT = qw(
   minimum_version_ok
   all_minimum_version_ok
   all_minimum_version_from_metayml_ok
+  all_minimum_version_from_metajson_ok
 );
 
 sub import {
@@ -58,15 +42,6 @@ sub _objectify_version {
            : version->new($version);
 }
 
-=head2 minimum_version_ok
-
-  minimum_version_ok($file, $version);
-
-This test passes if the given file does not seem to require any version of perl
-newer than C<$version>, which may be given as a version string or a version
-object.
-
-=cut
 
 sub minimum_version_ok {
   my ($file, $version) = @_;
@@ -77,8 +52,16 @@ sub minimum_version_ok {
 
   my $pmv = Perl::MinimumVersion->new($file);
 
-  my $explicit_minimum = $pmv->minimum_explicit_version;
-  my $minimum = $pmv->minimum_syntax_version($explicit_minimum);
+  unless (defined $pmv) {
+    $Test->ok(0, $file);
+    $Test->diag(
+      "$file could not be parsed: " . PPI::Document->errstr
+    );
+    return;
+  }
+
+  my $explicit_minimum = $pmv->minimum_explicit_version || 0;
+  my $minimum = $pmv->minimum_syntax_version($explicit_minimum) || 0;
 
   my $is_syntax = 1
     if $minimum and $minimum > $explicit_minimum;
@@ -88,9 +71,7 @@ sub minimum_version_ok {
 
   my %min = $pmv->version_markers;
 
-  if (not defined $minimum) {
-    $Test->ok(1, $file);
-  } elsif ($minimum <= $version) {
+  if ($minimum <= $version) {
     $Test->ok(1, $file);
   } else {
     $Test->ok(0, $file);
@@ -106,29 +87,17 @@ sub minimum_version_ok {
   }
 }
 
-=head2 all_minimum_version_ok
-
-  all_minimum_version_ok($version, \%arg);
-
-Given either a version string or a L<version> object, this routine produces a
-test plan (if there is no plan) and tests each relevant file with
-C<minimum_version_ok>.
-
-Relevant files are found by L<File::Find::Rule::Perl>.
-
-C<\%arg> is optional.  Valid arguments are:
-
-  paths   - in what paths to look for files; defaults to (t, lib, xt/smoke,
-            and any .pm or .PL files in the current working directory)
-            if it contains files, they will be checked
-  no_plan - do not plan the tests about to be run
-
-=cut
 
 sub all_minimum_version_ok {
   my ($version, $arg) = @_;
   $arg ||= {};
-  $arg->{paths} ||= [ qw(lib t xt/smoke), glob ("*.pm"), glob ("*.PL") ];
+  $arg->{paths} ||= [
+    qw(bin script lib t xt/smoke),
+    glob("*.pm"),
+    glob("*.PL"),
+  ];
+
+  $arg->{skip} ||= [];
 
   my $Test = Test::Builder->new;
 
@@ -143,6 +112,9 @@ sub all_minimum_version_ok {
     }
   }
 
+  my %skip = map {; $_ => 1 } @{ $arg->{skip} };
+  @perl_files = grep {; ! $skip{$_} } @perl_files;
+
   unless ($Test->has_plan or $arg->{no_plan}) {
     $Test->plan(tests => scalar @perl_files);
   }
@@ -150,6 +122,90 @@ sub all_minimum_version_ok {
   minimum_version_ok($_, $version) for @perl_files;
 }
 
+
+sub __version_from_meta {
+  my ($fn) = @_;
+
+  my $meta = CPAN::Meta->load_file($fn, { lazy_validation => 1 })->as_struct;
+  my $version = $meta->{prereqs}{runtime}{requires}{perl};
+}
+
+sub __from_meta {
+  my ($fn, $arg) = @_;
+  $arg ||= {};
+
+  my $Test = Test::Builder->new;
+
+  $Test->plan(skip_all => "$fn could not be found")
+    unless -f $fn and -r _;
+
+  $Test->plan(skip_all => "no minimum perl version could be determined")
+    unless my $version = __version_from_meta($fn);
+
+  all_minimum_version_ok($version, $arg);
+}
+
+sub all_minimum_version_from_metayml_ok {
+  __from_meta('META.yml', @_);
+}
+
+
+sub all_minimum_version_from_metajson_ok { __from_meta('META.json', @_); }
+
+1;
+
+__END__
+
+=pod
+
+=encoding UTF-8
+
+=head1 NAME
+
+Test::MinimumVersion - does your code require newer perl than you think?
+
+=head1 VERSION
+
+version 0.101081
+
+=head1 SYNOPSIS
+
+Example F<minimum-perl.t>:
+
+  #!perl
+  use Test::MinimumVersion;
+  all_minimum_version_ok('5.008');
+
+=head1 FUNCTIONS
+
+=head2 minimum_version_ok
+
+  minimum_version_ok($file, $version);
+
+This test passes if the given file does not seem to require any version of perl
+newer than C<$version>, which may be given as a version string or a version
+object.
+
+=head2 all_minimum_version_ok
+
+  all_minimum_version_ok($version, \%arg);
+
+Given either a version string or a L<version> object, this routine produces a
+test plan (if there is no plan) and tests each relevant file with
+C<minimum_version_ok>.
+
+Relevant files are found by L<File::Find::Rule::Perl>.
+
+C<\%arg> is optional.  Valid arguments are:
+
+  paths   - in what paths to look for files; defaults to (bin, script, t, lib,
+            xt/smoke, and any .pm or .PL files in the current working
+            directory) if it contains files, they will be checked
+  no_plan - do not plan the tests about to be run
+  skip    - files to skip; this can be useful in weird cases like gigantic
+            files, files falsely detected as Perl, or code that uses
+            a source filter; this should be an arrayref of filenames
+
 =head2 all_minimum_version_from_metayml_ok
 
   all_minimum_version_from_metayml_ok(\%arg);
@@ -159,40 +215,24 @@ META.yml file or no perl version is found, all tests are skipped.  If a version
 is found, the test proceeds as if C<all_minimum_version_ok> had been called
 with that version.
 
-=cut
+=head2 all_minimum_version_from_metajson_ok
 
-sub all_minimum_version_from_metayml_ok {
-  my ($arg) = @_;
-  $arg ||= {};
+  all_minimum_version_from_metajson_ok(\%arg);
 
-  my $Test = Test::Builder->new;
-
-  $Test->plan(skip_all => "META.yml could not be found")
-    unless -f 'META.yml' and -r _;
-
-  my $documents = YAML::Tiny->read('META.yml');
-
-  $Test->plan(skip_all => "no minimum perl version could be determined")
-    unless my $version = $documents->[0]->{requires}{perl};
-
-  all_minimum_version_ok($version, $arg);
-}
-
-=head1 TODO
-
-Uh, this code has no tests.  I'm really sorry.  I'll get around to it.
+This routine checks F<META.json> for an entry in F<requires> for F<perl>.  If
+no META.json file or no perl version is found, all tests are skipped.  If a
+version is found, the test proceeds as if C<all_minimum_version_ok> had been
+called with that version.
 
 =head1 AUTHOR
 
-Ricardo SIGNES, C<< <rjbs at cpan.org> >>
+Ricardo Signes
 
-=head1 COPYRIGHT & LICENSE
+=head1 COPYRIGHT AND LICENSE
 
-Copyright 2007, Ricardo SIGNES.
+This software is copyright (c) 2007 by Ricardo Signes.
 
-This program is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
 =cut
-
-1;
